@@ -120,10 +120,6 @@ typedef shared_ptr<class_diff> class_diff_sptr;
 /// associated to the first one.
 typedef unordered_map<size_t, size_t> pointer_map;
 
-/// Convenience typedef for a map which key is a string and which
-/// value is a @ref decl_base_sptr.
-typedef unordered_map<string, decl_base_sptr> string_decl_base_sptr_map;
-
 /// Convenience typedef for a map which key is a string and which
 /// value is a @ref type_base_sptr.
 typedef unordered_map<string, type_base_sptr> string_type_base_sptr_map;
@@ -204,7 +200,7 @@ typedef unordered_map<string, changed_enumerator> string_changed_enumerator_map;
 
 /// Convenience typedef for a map which key is a string and which
 /// value is a pointer to @ref decl_base.
-typedef unordered_map<string, function_decl*> string_function_ptr_map;
+typedef unordered_map<string, const function_decl*> string_function_ptr_map;
 
 /// Convenience typedef for a map which key is a string and which
 /// value is a @ref function_decl_diff_sptr.
@@ -229,7 +225,7 @@ typedef unordered_map<string, method_decl_sptr> string_member_function_sptr_map;
 
 /// Convenience typedef for a map which key is a string and which
 /// value is a point to @ref var_decl.
-typedef unordered_map<string, var_decl*> string_var_ptr_map;
+typedef unordered_map<string, const var_decl*> string_var_ptr_map;
 
 /// Convenience typedef for a pair of pointer to @ref var_decl
 /// representing a @ref var_decl change.  The first member of the pair
@@ -357,8 +353,8 @@ enum diff_category
   HARMLESS_SYMBOL_ALIAS_CHANGE_CATEGORY = 1 << 6,
 
   /// This means that a diff node in the sub-tree carries a harmless
-  /// union change.
-  HARMLESS_UNION_CHANGE_CATEGORY = 1 << 7,
+  /// union or class change.
+  HARMLESS_UNION_OR_CLASS_CHANGE_CATEGORY = 1 << 7,
 
   /// This means that a diff node in the sub-tree carries a harmless
   /// data member change.  An example of harmless data member change
@@ -455,7 +451,7 @@ enum diff_category
   | STATIC_DATA_MEMBER_CHANGE_CATEGORY
   | HARMLESS_ENUM_CHANGE_CATEGORY
   | HARMLESS_SYMBOL_ALIAS_CHANGE_CATEGORY
-  | HARMLESS_UNION_CHANGE_CATEGORY
+  | HARMLESS_UNION_OR_CLASS_CHANGE_CATEGORY
   | HARMLESS_DATA_MEMBER_CHANGE_CATEGORY
   | SUPPRESSED_CATEGORY
   | PRIVATE_TYPE_CATEGORY
@@ -1416,6 +1412,68 @@ compute_diff(reference_type_def_sptr first,
 	     diff_context_sptr ctxt);
 
 
+class ptr_to_mbr_diff;
+
+/// Typedef of a shared_ptr to @ref ptr_to_mbr_diff
+typedef shared_ptr<ptr_to_mbr_diff> ptr_to_mbr_diff_sptr;
+
+/// The abstraction of a diff between two @ref ptr_to_mbr_type.
+class ptr_to_mbr_diff : public type_diff_base
+{
+  struct priv;
+  std::unique_ptr<priv> priv_;
+
+  ptr_to_mbr_diff()  = default;
+
+protected:
+  ptr_to_mbr_diff(const ptr_to_mbr_type_sptr& first,
+		  const ptr_to_mbr_type_sptr& second,
+		  const diff_sptr&	      member_type_diff,
+		  const diff_sptr&	      containing_type_diff,
+		  diff_context_sptr	      ctxt);
+
+public:
+
+  ptr_to_mbr_type_sptr
+  first_ptr_to_mbr_type() const;
+
+  ptr_to_mbr_type_sptr
+  second_ptr_to_mbr_type() const;
+
+  const diff_sptr
+  member_type_diff() const;
+
+  const diff_sptr
+  containing_type_diff() const;
+
+  virtual bool
+  has_changes() const;
+
+  virtual enum change_kind
+  has_local_changes() const;
+
+  virtual const string&
+  get_pretty_representation() const;
+
+  virtual void
+  report(ostream&, const string& indent = "") const;
+
+  virtual void
+  chain_into_hierarchy();
+
+  virtual ~ptr_to_mbr_diff();
+
+  friend ptr_to_mbr_diff_sptr
+  compute_diff(const ptr_to_mbr_type_sptr& first,
+	       const ptr_to_mbr_type_sptr& second,
+	       diff_context_sptr&		  ctxt);
+}; // end class ptr_to_mbr_diff
+
+ptr_to_mbr_diff_sptr
+compute_diff(const ptr_to_mbr_type_sptr& first,
+	     const ptr_to_mbr_type_sptr& second,
+	     diff_context_sptr&	 ctxt);
+
 class subrange_diff;
 
 /// A convenience typedef for a shared pointer to subrange_diff type.
