@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2017-2022 Red Hat, Inc.
+// Copyright (C) 2017-2023 Red Hat, Inc.
 //
 // Author: Dodji Seketeli
 
@@ -32,6 +32,8 @@ class qualified_type_diff;
 class distinct_diff;
 class pointer_diff;
 class reference_diff;
+class ptr_to_mbr_diff;
+class subrange_diff;
 class array_diff;
 class base_diff;
 class class_or_union_diff;
@@ -88,10 +90,18 @@ public:
   report(const reference_diff& d, std::ostream& out,
 	 const std::string& indent = "") const = 0;
 
+  virtual void
+  report(const ptr_to_mbr_diff& d, std::ostream& out,
+	 const std::string& indent = "") const = 0;
+
   virtual void
   report(const array_diff& d, std::ostream& out,
 	 const std::string& indent = "") const = 0;
 
+  virtual void
+  report(const subrange_diff& d, std::ostream& out,
+	 const std::string& indent = "") const = 0;
+
   virtual void
   report(const base_diff& d, std::ostream& out,
 	 const std::string& indent = "") const = 0;
@@ -173,6 +183,11 @@ public:
 				      std::ostream& out,
 				      const std::string& indent) const;
 
+  void
+  report_underlying_changes_of_qualified_type (const qualified_type_diff& d,
+					       ostream& out,
+					       const string& indent) const;
+
   virtual void
   report(const qualified_type_diff& d, std::ostream& out,
 	 const std::string& indent = "") const;
@@ -190,6 +205,15 @@ public:
   report(const reference_diff& d, std::ostream& out,
 	 const std::string& indent = "") const;
 
+  bool
+  report_local_ptr_to_mbr_type_changes(const ptr_to_mbr_diff& d,
+				       std::ostream& out,
+				       const std::string& indent = "") const;
+
+  virtual void
+  report(const ptr_to_mbr_diff& d, std::ostream& out,
+	 const std::string& indent = "") const;
+
   virtual void
   report(const fn_parm_diff& d, std::ostream& out,
 	 const std::string& indent = "") const;
@@ -203,6 +227,10 @@ public:
   report(const function_type_diff& d, std::ostream& out,
 	 const std::string& indent = "") const;
 
+  virtual void
+  report(const subrange_diff& d, std::ostream& out,
+	 const std::string& indent = "") const;
+
   virtual void
   report(const array_diff& d, std::ostream& out,
 	 const std::string& indent = "") const;
@@ -277,6 +305,10 @@ public:
   report(const reference_diff& d, std::ostream& out,
 	 const std::string& indent = "") const;
 
+  virtual void
+  report(const ptr_to_mbr_diff& d, std::ostream& out,
+	 const std::string& indent = "") const;
+
   virtual void
   report(const fn_parm_diff& d, std::ostream& out,
 	 const std::string& indent = "") const;
@@ -285,6 +317,10 @@ public:
   report(const function_type_diff& d, std::ostream& out,
 	 const std::string& indent = "") const;
 
+  virtual void
+  report(const subrange_diff& d, std::ostream& out,
+	 const std::string& indent = "") const;
+
   virtual void
   report(const array_diff& d, std::ostream& out,
 	 const std::string& indent = "") const;
