@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2016-2022 Red Hat, Inc.
+// Copyright (C) 2016-2023 Red Hat, Inc.
 //
 // Author: Dodji Seketeli
 
@@ -11,7 +11,7 @@
 #include <unordered_set>
 
 #include "abg-ini.h"
-#include "abg-comparison.h"
+#include "abg-ir.h"
 
 namespace abigail
 {
@@ -29,15 +29,24 @@ class fe_iface;
 /// that are defined in this namespace.
 namespace suppr
 {
-
-using namespace abigail::comparison;
 using std::unordered_set;
+using std::string;
+using std::shared_ptr;
+using std::vector;
+using comparison::diff;
+using comparison::diff_context_sptr;
 
-/// Base type of the suppression specifications types.
+/// Base type of a direct suppression specifications types.
 ///
 /// This abstracts a suppression specification.  It's a way to specify
 /// how to drop reports about a particular diff node on the floor, if
 /// it matches the supppression specification.
+///
+/// Note that a direct suppression specification suppresses (for
+/// reporting purposes) the diff node that it matches.  A negated
+/// suppression specification, however, suppresses a diff node that it
+/// DOES NOT match.  A Negated suppression specification is abstracted
+/// by the class @ref negated_suppression_base.
 class suppression_base
 {
 public:
@@ -140,6 +149,36 @@ typedef shared_ptr<type_suppression> type_suppression_sptr;
 /// Convenience typedef for vector of @ref type_suppression_sptr.
 typedef vector<type_suppression_sptr> type_suppressions_type;
 
+/// The base class of suppression specifications that are defined by
+/// the negation of matching clauses.
+///
+/// A direct suppression specification suppresses (for reporting
+/// purposes) the diff node that it matches.  A negated suppression
+/// specification suppresses a diff node that it DOES NOT match.
+class negated_suppression_base
+{
+public:
+  negated_suppression_base();
+
+  virtual ~negated_suppression_base();
+}; // end class negated_suppression_base.
+
+/// A convenience typedef for a shared pointer to @ref
+/// negated_suppression_base.
+typedef shared_ptr<negated_suppression_base> negated_suppression_sptr;
+
+/// Convenience typedef for a vector of @ref negated_suppression_sptr
+typedef vector<negated_suppression_sptr> negated_suppressions_type;
+
+bool
+is_negated_suppression(const suppression_base&);
+
+const negated_suppression_base*
+is_negated_suppression(const suppression_base*);
+
+negated_suppression_sptr
+is_negated_suppression(const suppression_sptr&);
+
 /// Abstraction of a type suppression specification.
 ///
 /// Specifies under which condition reports about a type diff node
@@ -243,6 +282,24 @@ public:
   void
   set_reach_kind(reach_kind k);
 
+  bool
+  get_has_size_change() const;
+
+  void
+  set_has_size_change(bool flag);
+
+  const string_set_type&
+  get_potential_data_member_names() const;
+
+  void
+  set_potential_data_member_names(const string_set_type&) const;
+
+  const string&
+  get_potential_data_member_names_regex_str() const;
+
+  void
+  set_potential_data_member_names_regex_str(const string&) const;
+
   void
   set_data_member_insertion_ranges(const insertion_ranges& r);
 
@@ -273,6 +330,18 @@ public:
   void
   set_changed_enumerator_names(const vector<string>&);
 
+  const vector<regex::regex_t_sptr>&
+  get_changed_enumerators_regexp() const;
+
+  void
+  set_changed_enumerators_regexp(const vector<regex::regex_t_sptr>&);
+
+  bool
+  has_strict_fam_conversion () const;
+
+  void
+  set_has_strict_fam_conversion(bool);
+
   virtual bool
   suppresses_diff(const diff* diff) const;
 
@@ -303,6 +372,7 @@ public:
   class boundary;
   class integer_boundary;
   class fn_call_expr_boundary;
+  class named_boundary;
 
   /// Convenience typedef for a shared_ptr to @ref boundary
   typedef shared_ptr<boundary> boundary_sptr;
@@ -314,6 +384,10 @@ public:
   /// fn_call_expr_boundary
   typedef shared_ptr<fn_call_expr_boundary> fn_call_expr_boundary_sptr;
 
+  /// Convenience typedef for a shared_ptr to a @ref
+  /// named_boundary
+  typedef shared_ptr<named_boundary> named_boundary_sptr;
+
   insertion_range();
 
   insertion_range(boundary_sptr begin, boundary_sptr end);
@@ -321,7 +395,7 @@ public:
   boundary_sptr
   begin() const;
 
- boundary_sptr
+  boundary_sptr
   end() const;
 
   static insertion_range::integer_boundary_sptr
@@ -333,10 +407,13 @@ public:
   static insertion_range::fn_call_expr_boundary_sptr
   create_fn_call_expr_boundary(const string&);
 
+  static insertion_range::named_boundary_sptr
+  create_named_boundary(const string&);
+
   static bool
-  eval_boundary(boundary_sptr	boundary,
-		class_decl_sptr context,
-		uint64_t&	value);
+  eval_boundary(const boundary_sptr	boundary,
+		const class_or_union*	context,
+		uint64_t&		value);
 
   static bool
   boundary_value_is_end(uint64_t value);
@@ -348,6 +425,9 @@ is_integer_boundary(type_suppression::insertion_range::boundary_sptr);
 type_suppression::insertion_range::fn_call_expr_boundary_sptr
 is_fn_call_expr_boundary(type_suppression::insertion_range::boundary_sptr);
 
+type_suppression::insertion_range::named_boundary_sptr
+is_named_boundary(type_suppression::insertion_range::boundary_sptr);
+
 /// The abstraction of the boundary of an @ref insertion_range, in the
 /// context of a @ref type_suppression
 class type_suppression::insertion_range::boundary
@@ -395,6 +475,53 @@ public:
   ~fn_call_expr_boundary();
 }; //end class type_suppression::insertion_range::fn_call_expr_boundary
 
+/// An @ref insertion_range boundary that is expressed as a named
+/// constant that is to be evaluated later in the context of a given
+/// type and resolved to a bit offset.
+class type_suppression::insertion_range::named_boundary
+  : public type_suppression::insertion_range::boundary
+{
+  struct priv;
+  std::unique_ptr<priv> priv_;
+
+  named_boundary();
+
+public:
+  named_boundary(const string& name);
+  const string& get_name() const;
+}; //end class type_suppression::insertion_range::named_boundary
+
+/// Abstraction of a negated type suppression specification.
+///
+/// A negated type suppression suppresses a type if the negation of
+/// the equivalent propositions for a @ref type_suppression are valid.
+class negated_type_suppression : virtual public type_suppression,
+				 virtual public negated_suppression_base
+{
+
+public:
+
+  negated_type_suppression(const string& label,
+			   const string& type_name_regexp,
+			   const string& type_name);
+
+  virtual bool
+  suppresses_diff(const diff* diff) const;
+
+  bool
+  suppresses_type(const type_base_sptr& type,
+		  const diff_context_sptr& ctxt) const;
+
+  bool
+  suppresses_type(const type_base_sptr& type) const;
+
+  bool
+  suppresses_type(const type_base_sptr& type,
+		  const scope_decl* type_scope) const;
+
+  virtual ~negated_type_suppression();
+};// end class negated_type_suppression
+
 class function_suppression;
 
 /// Convenience typedef for a shared pointer to function_suppression.
@@ -822,13 +949,13 @@ suppression_matches_soname_or_filename(const string& soname,
 				       const suppression_base& suppr);
 
 const char*
-get_private_types_suppr_spec_label();
+get_opaque_types_suppr_spec_label();
 
 bool
-is_private_type_suppr_spec(const type_suppression&);
+is_opaque_type_suppr_spec(const type_suppression&);
 
 bool
-is_private_type_suppr_spec(const suppression_sptr& s);
+is_opaque_type_suppr_spec(const suppression_sptr& s);
 
 bool
 suppression_can_match(const fe_iface&,
@@ -887,6 +1014,12 @@ is_type_suppressed(const fe_iface&	fe,
 		   const location&	type_location,
 		   bool&		type_is_private,
 		   bool			require_drop_property = false);
+
+bool
+is_data_member_offset_in_range(const var_decl_sptr&,
+			       const type_suppression::insertion_range_sptr&,
+			       const class_or_union*);
+
 } // end namespace suppr
 
 
