@@ -397,7 +397,8 @@ public:
       }
 
     canonicalize_types();
-
+    corpus()->sort_functions();
+    corpus()->sort_variables();
     return corpus();
   }
 
@@ -504,15 +505,9 @@ public:
     associate_artifact_to_btf_type_id(result, type_id);
 
     if (function_decl_sptr fn = is_function_decl(result))
-      {
-	if (fn->get_is_in_public_symbol_table())
-	  maybe_add_fn_to_exported_decls(fn.get());
-      }
+      add_fn_to_exported_or_undefined_decls(fn.get());
     else if (var_decl_sptr var = is_var_decl(result))
-      {
-	if (var->get_is_in_public_symbol_table())
-	  maybe_add_var_to_exported_decls(var.get());
-      }
+      add_var_to_exported_or_undefined_decls(var.get());
 
     return result;
   }
@@ -789,14 +784,15 @@ public:
     if (!underlying_type)
       return type_or_decl_base_sptr();
 
-    uint64_t lower_boud = 0;
+    uint64_t lower_bound = 0;
     // Note that arr->nelems can be 0;
     uint64_t upper_bound = arr->nelems ? arr->nelems - 1: 0;
 
     array_type_def::subrange_sptr subrange(new array_type_def::subrange_type
 					   (env(), /*name=*/"",
-					    lower_boud, upper_bound,
+					    lower_bound, upper_bound,
 					    location()));
+    subrange->is_non_finite(!arr->nelems);
     add_decl_to_scope(subrange, cur_tu()->get_global_scope());
     canonicalize(subrange);
     array_type_def::subranges_type subranges = {subrange};
@@ -1020,10 +1016,12 @@ public:
 				   location(), /*linkage_name=*/fn_name));
 
     elf_symbol_sptr fn_sym;
-    if ((fn_sym = function_symbol_is_exported(fn_name)))
+    if ((fn_sym = function_symbol_is_exported(fn_name))
+	|| (fn_sym = function_symbol_is_undefined(fn_name)))
       {
 	result->set_symbol(fn_sym);
-	result->set_is_in_public_symbol_table(true);
+	if (fn_sym->is_defined())
+	  result->set_is_in_public_symbol_table(true);
       }
     return result;
   }
@@ -1054,10 +1052,12 @@ public:
 			      /*linkage_name=*/var_name));
 
     elf_symbol_sptr var_sym;
-    if ((var_sym = variable_symbol_is_exported(var_name)))
+    if ((var_sym = variable_symbol_is_exported(var_name))
+	|| (var_sym = variable_symbol_is_undefined(var_name)))
       {
 	result->set_symbol(var_sym);
-	result->set_is_in_public_symbol_table(true);
+	if (var_sym->is_defined())
+	  result->set_is_in_public_symbol_table(true);
       }
     return result;
   }
