@@ -479,7 +479,7 @@ public:
 	    add_decl_to_scope(var_declaration,
 			      ir_translation_unit->get_global_scope());
 	    var_declaration->set_is_in_public_symbol_table(true);
-	    maybe_add_var_to_exported_decls(var_declaration.get());
+	    add_var_to_exported_or_undefined_decls(var_declaration.get());
 	  }
 	else
 	  {
@@ -499,7 +499,7 @@ public:
 	    add_decl_to_scope(func_declaration,
 			      ir_translation_unit->get_global_scope());
 	    func_declaration->set_is_in_public_symbol_table(true);
-	    maybe_add_fn_to_exported_decls(func_declaration.get());
+	    add_fn_to_exported_or_undefined_decls(func_declaration.get());
 	  }
 
 	ctf_dict = dict_tmp;
@@ -511,6 +511,8 @@ public:
        require other related types to not be already
        canonicalized.  */
     canonicalize_all_types();
+    corpus()->sort_functions();
+    corpus()->sort_variables();
   }
 
   /// Add a new type declaration to the given libabigail IR corpus CORP.
@@ -1231,7 +1233,7 @@ static array_type_def::subrange_sptr
 build_array_ctf_range(reader *rdr, ctf_dict_t *dic,
                       ctf_id_t index, uint64_t nelems)
 {
-  bool is_infinite = false;
+  bool is_non_finite = false;
   corpus_sptr corp = rdr->corpus();
   translation_unit_sptr tunit = rdr->cur_transl_unit();
   array_type_def::subrange_sptr subrange;
@@ -1247,7 +1249,7 @@ build_array_ctf_range(reader *rdr, ctf_dict_t *dic,
 
   /* for VLAs number of array elements is 0 */
   if (upper_bound.get_unsigned_value() == 0 && nelems == 0)
-    is_infinite = true;
+    is_non_finite = true;
 
   subrange.reset(new array_type_def::subrange_type(rdr->env(),
                                                    "",
@@ -1259,7 +1261,7 @@ build_array_ctf_range(reader *rdr, ctf_dict_t *dic,
   if (!subrange)
     return nullptr;
 
-  subrange->is_infinite(is_infinite);
+  subrange->is_non_finite(is_non_finite);
   add_decl_to_scope(subrange, tunit->get_global_scope());
   canonicalize(subrange);
 
