@@ -788,13 +788,12 @@ reader::symtab() const
 elf_symbol_sptr
 reader::function_symbol_is_exported(GElf_Addr symbol_address) const
 {
-  elf_symbol_sptr symbol = symtab()->lookup_symbol(symbol_address);
+
+  elf_symbol_sptr symbol =
+    symtab()->function_symbol_is_exported(symbol_address);
   if (!symbol)
     return symbol;
 
-  if (!symbol->is_function() || !symbol->is_public())
-    return elf_symbol_sptr();
-
   address_set_sptr set;
   bool looking_at_linux_kernel_binary =
     load_in_linux_kernel_mode() && elf_helpers::is_linux_kernel(elf_handle());
@@ -820,13 +819,11 @@ reader::function_symbol_is_exported(GElf_Addr symbol_address) const
 elf_symbol_sptr
 reader::variable_symbol_is_exported(GElf_Addr symbol_address) const
 {
-  elf_symbol_sptr symbol = symtab()->lookup_symbol(symbol_address);
+  elf_symbol_sptr symbol =
+    symtab()->variable_symbol_is_exported(symbol_address);
   if (!symbol)
     return symbol;
 
-  if (!symbol->is_variable() || !symbol->is_public())
-    return elf_symbol_sptr();
-
   address_set_sptr set;
   bool looking_at_linux_kernel_binary =
     load_in_linux_kernel_mode() && elf_helpers::is_linux_kernel(elf_handle());
@@ -849,23 +846,20 @@ reader::variable_symbol_is_exported(GElf_Addr symbol_address) const
 elf_symbol_sptr
 reader::function_symbol_is_exported(const string& name) const
 {
-  const elf_symbols& syms = symtab()->lookup_symbol(name);
-  for (auto s : syms)
+  const elf_symbol_sptr s = symtab()->function_symbol_is_exported(name);
+  if (s && s->is_function() && s->is_public())
     {
-      if (s->is_function() && s->is_public())
-	{
-	  bool looking_at_linux_kernel_binary =
-	    (load_in_linux_kernel_mode()
-	     && elf_helpers::is_linux_kernel(elf_handle()));
+      bool looking_at_linux_kernel_binary =
+	(load_in_linux_kernel_mode()
+	 && elf_helpers::is_linux_kernel(elf_handle()));
 
-	  if (looking_at_linux_kernel_binary)
-	    {
-	      if (s->is_in_ksymtab())
-		return s;
-	    }
-	  else
+      if (looking_at_linux_kernel_binary)
+	{
+	  if (s->is_in_ksymtab())
 	    return s;
 	}
+      else
+	return s;
     }
   return elf_symbol_sptr();
 }
@@ -879,26 +873,42 @@ reader::function_symbol_is_exported(const string& name) const
 elf_symbol_sptr
 reader::variable_symbol_is_exported(const string& name) const
 {
-  const elf_symbols& syms = symtab()->lookup_symbol(name);
-  for (auto s : syms)
+  const elf_symbol_sptr s  = symtab()->variable_symbol_is_exported(name);
+  if (s->is_variable() && s->is_public())
     {
-      if (s->is_variable() && s->is_public())
-	{
-	  bool looking_at_linux_kernel_binary =
-	    (load_in_linux_kernel_mode()
-	     && elf_helpers::is_linux_kernel(elf_handle()));
+      bool looking_at_linux_kernel_binary =
+	(load_in_linux_kernel_mode()
+	 && elf_helpers::is_linux_kernel(elf_handle()));
 
-	  if (looking_at_linux_kernel_binary)
-	    {
-	      if (s->is_in_ksymtab())
-		return s;
-	    }
-	  else
+      if (looking_at_linux_kernel_binary)
+	{
+	  if (s->is_in_ksymtab())
 	    return s;
 	}
+      else
+	return s;
     }
   return elf_symbol_sptr();
 }
+
+/// Test if a name is the name of an undefined function symbol.
+///
+/// @param name the symbol name to consider.
+///
+/// @return the undefined function symbol or nil if none was found.
+elf_symbol_sptr
+reader::function_symbol_is_undefined(const string& name) const
+{return symtab()->function_symbol_is_undefined(name);}
+
+/// Test if a name is the name of an undefined variable symbol.
+///
+/// @param name the symbol name to consider.
+///
+/// @return the undefined variable symbol or nil if none was found.
+elf_symbol_sptr
+reader::variable_symbol_is_undefined(const string& name) const
+{return symtab()->variable_symbol_is_undefined(name);}
+
 /// Load the DT_NEEDED and DT_SONAME elf TAGS.
 void
 reader::load_dt_soname_and_needed()
