@@ -391,6 +391,7 @@ struct environment::priv
   canonical_types_map_type		canonical_types_;
   mutable vector<type_base_sptr>	sorted_canonical_types_;
   type_base_sptr			void_type_;
+  type_base_sptr			void_pointer_type_;
   type_base_sptr			variadic_marker_type_;
   // The set of pairs of class types being currently compared.  It's
   // used to avoid endless loops while recursively comparing types.
@@ -493,6 +494,7 @@ struct environment::priv
   bool					decl_only_class_equals_definition_;
   bool					use_enum_binary_only_equality_;
   bool					allow_type_comparison_results_caching_;
+  bool					do_log_;
   optional<bool>			analyze_exported_interfaces_only_;
 #ifdef WITH_DEBUG_SELF_COMPARISON
   bool					self_comparison_debug_on_;
@@ -519,7 +521,8 @@ struct environment::priv
       do_on_the_fly_canonicalization_(true),
       decl_only_class_equals_definition_(false),
       use_enum_binary_only_equality_(true),
-      allow_type_comparison_results_caching_(false)
+      allow_type_comparison_results_caching_(false),
+      do_log_(false)
 #ifdef WITH_DEBUG_SELF_COMPARISON
     ,
       self_comparison_debug_on_(false)
@@ -552,6 +555,14 @@ struct environment::priv
   allow_type_comparison_results_caching() const
   {return allow_type_comparison_results_caching_;}
 
+  void
+  do_log(bool f)
+  {do_log_ = f;}
+
+  bool
+  do_log() const
+  {return do_log_;}
+
   /// Cache the result of comparing two sub-types.
   ///
   /// @param first the first sub-type that has been compared. Its
@@ -1052,6 +1063,7 @@ struct environment::priv
     // tentative canonical type that might be later canceled.
     t->priv_->set_does_not_depend_on_recursive_type();
     env.priv_->remove_from_types_with_non_confirmed_propagated_ct(t);
+    env.priv_->clear_type_comparison_results_cache();
   }
 
   /// Clear the propagated canonical type of a given type.
@@ -1318,9 +1330,16 @@ canonicalize_types(const input_iterator& begin,
   if (begin == end)
     return;
 
+  int i;
+  input_iterator t;
   // First, let's compute the canonical type of this type.
-  for (auto t = begin; t != end; ++t)
-    canonicalize(deref(t));
+  for (t = begin,i = 0; t != end; ++t, ++i)
+    {
+      if (deref(t)->get_environment().priv_->do_log())
+	std::cerr << "#" << std::dec << i << " ";
+
+      canonicalize(deref(t));
+    }
 
 #ifdef WITH_DEBUG_CT_PROPAGATION
   // Then now, make sure that all types -- which propagated canonical
