@@ -1185,6 +1185,231 @@ InOutSpec in_out_specs[] =
     "data/test-abidiff-exit/test-fam2-report-1.txt",
     "output/test-abidiff-exit/test-fam2-report-1.txt"
   },
+  {
+    "data/test-abidiff-exit/PR31377/reference/lib64/librte_pipeline.so.24.0",
+    "data/test-abidiff-exit/PR31377/flex/lib64/librte_pipeline.so.24.1",
+    "",
+    "data/test-abidiff-exit/PR31377/reference/include",
+    "data/test-abidiff-exit/PR31377/flex/include",
+    "",
+    "",
+    "",
+    "",
+    "--no-default-suppression --no-added-syms",
+    abigail::tools_utils::ABIDIFF_OK,
+    "data/test-abidiff-exit/PR31377/test-PR31377-report-1.txt",
+    "output/test-abidiff-exit/PR31377/test-PR31377-report-1.txt"
+  },
+  {
+    "data/test-abidiff-exit/PR31045/zfs-abigail-2.4/libnvpair.abi",
+    "data/test-abidiff-exit/PR31045/zfs-abigail-2.4/libnvpair.so",
+    "data/test-abidiff-exit/PR31045/zfs-abigail-2.4/libnvpair.suppr",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "--no-default-suppression",
+    abigail::tools_utils::ABIDIFF_OK,
+    "data/test-abidiff-exit/PR31045/zfs-abigail-2.4/test-PR31045-report-1.txt",
+    "output/test-abidiff-exit/PR31045/zfs-abigail-2.4/test-PR31045-report-1.txt"
+  },
+  {
+    "data/test-abidiff-exit/pointer-to-member/test-ptr-to-mbr1-v0.o",
+    "data/test-abidiff-exit/pointer-to-member/test-ptr-to-mbr1-v1.o",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "--no-default-suppression",
+    abigail::tools_utils::ABIDIFF_ABI_CHANGE,
+    "data/test-abidiff-exit/pointer-to-member/test-ptr-to-mbr1-output-1.txt",
+    "output/test-abidiff-exit/pointer-to-member/test-ptr-to-mbr1-output-1.txt"
+  },
+  {
+    "data/test-abidiff-exit/pointer-to-member/test-ptr-to-mbr2-v0.o",
+    "data/test-abidiff-exit/pointer-to-member/test-ptr-to-mbr2-v1.o",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "--no-default-suppression",
+    abigail::tools_utils::ABIDIFF_ABI_CHANGE,
+    "data/test-abidiff-exit/pointer-to-member/test-ptr-to-mbr2-output-1.txt",
+    "output/test-abidiff-exit/pointer-to-member/test-ptr-to-mbr2-output-1.txt"
+  },
+  {
+    "data/test-abidiff-exit/pointer-to-member/test-ptr-to-mbr3-v0.o",
+    "data/test-abidiff-exit/pointer-to-member/test-ptr-to-mbr3-v1.o",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "--no-default-suppression",
+    abigail::tools_utils::ABIDIFF_ABI_CHANGE,
+    "data/test-abidiff-exit/pointer-to-member/test-ptr-to-mbr3-output-1.txt",
+    "output/test-abidiff-exit/pointer-to-member/test-ptr-to-mbr3-output-1.txt"
+  },
+  {
+    "data/test-abidiff-exit/pointer-to-member/test-ptr-to-mbr4-v0.o",
+    "data/test-abidiff-exit/pointer-to-member/test-ptr-to-mbr4-v1.o",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "--no-default-suppression",
+    abigail::tools_utils::ABIDIFF_ABI_CHANGE,
+    "data/test-abidiff-exit/pointer-to-member/test-ptr-to-mbr4-output-1.txt",
+    "output/test-abidiff-exit/pointer-to-member/test-ptr-to-mbr4-output-1.txt"
+  },
+  {
+    "data/test-abidiff-exit/pointer-to-member/test-ptr-to-mbr5-v0.o",
+    "data/test-abidiff-exit/pointer-to-member/test-ptr-to-mbr5-v1.o",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "--no-default-suppression",
+    abigail::tools_utils::ABIDIFF_OK,
+    "data/test-abidiff-exit/pointer-to-member/test-ptr-to-mbr5-output-1.txt",
+    "output/test-abidiff-exit/pointer-to-member/test-ptr-to-mbr5-output-1.txt"
+  },
+  {
+    "data/test-abidiff-exit/PR31513/reported/libloremipsum_gcc7.so",
+    "data/test-abidiff-exit/PR31513/reported/libloremipsum_gcc11.so",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "--no-default-suppression",
+    abigail::tools_utils::ABIDIFF_ABI_CHANGE | abigail::tools_utils::ABIDIFF_ABI_INCOMPATIBLE_CHANGE,
+    "data/test-abidiff-exit/PR31513/reported/PR31513-reported-report-1.txt",
+    "output/test-abidiff-exit/PR31513/reported/PR31513-reported-report-1.txt"
+  },
+  {
+    "data/test-abidiff-exit/PR31513/reported/libloremipsum_gcc7.so",
+    "data/test-abidiff-exit/PR31513/reported/libloremipsum_gcc11.so",
+    "data/test-abidiff-exit/PR31513/reported/libstdcpp.suppr",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "--no-default-suppression",
+    abigail::tools_utils::ABIDIFF_OK,
+    "data/test-abidiff-exit/PR31513/reported/PR31513-reported-report-2.txt",
+    "output/test-abidiff-exit/PR31513/reported/PR31513-reported-report-2.txt"
+  },
+  {
+    "data/test-abidiff-exit/PR31513/non-regr/libtest1-v0.so",
+    "data/test-abidiff-exit/PR31513/non-regr/libtest1-v1.so",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "--no-default-suppression",
+    abigail::tools_utils::ABIDIFF_OK,
+    "data/test-abidiff-exit/PR31513/non-regr/report1.txt",
+    "output/test-abidiff-exit/PR31513/non-regr/report1.txt"
+  },
+  {
+    "data/test-abidiff-exit/PR31513/non-regr/libtest2-v0.so",
+    "data/test-abidiff-exit/PR31513/non-regr/libtest2-v1.so",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "--no-default-suppression",
+    abigail::tools_utils::ABIDIFF_OK,
+    "data/test-abidiff-exit/PR31513/non-regr/report2.txt",
+    "output/test-abidiff-exit/PR31513/non-regr/report2.txt"
+  },
+  {
+    "data/test-abidiff-exit/PR31513/non-regr/libtest3-v0.so",
+    "data/test-abidiff-exit/PR31513/non-regr/libtest3-v1.so",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "--no-default-suppression",
+    abigail::tools_utils::ABIDIFF_ABI_CHANGE,
+    "data/test-abidiff-exit/PR31513/non-regr/report3.txt",
+    "output/test-abidiff-exit/PR31513/non-regr/report3.txt"
+  },
+  {
+    "data/test-abidiff-exit/PR31513/non-regr/libtest4-v0.so",
+    "data/test-abidiff-exit/PR31513/non-regr/libtest4-v1.so",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "--no-default-suppression",
+    abigail::tools_utils::ABIDIFF_ABI_CHANGE,
+    "data/test-abidiff-exit/PR31513/non-regr/report4.txt",
+    "output/test-abidiff-exit/PR31513/non-regr/report4.txt"
+  },
+  {
+    "data/test-abidiff-exit/non-del-anon-dm/reported/librte_graph.so.24.0",
+    "data/test-abidiff-exit/non-del-anon-dm/reported/librte_graph.so.24.1",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "--no-default-suppression",
+    abigail::tools_utils::ABIDIFF_OK,
+    "data/test-abidiff-exit/non-del-anon-dm/reported/report0.txt",
+    "output/test-abidiff-exit/non-del-anon-dm/reported/report0.txt"
+  },
+  {
+    "data/test-abidiff-exit/non-del-anon-dm/non-regr/test0-v0.o",
+    "data/test-abidiff-exit/non-del-anon-dm/non-regr/test0-v1.o",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "--no-default-suppression",
+    abigail::tools_utils::ABIDIFF_OK,
+    "data/test-abidiff-exit/non-del-anon-dm/non-regr/report0.txt",
+    "output/test-abidiff-exit/non-del-anon-dm/non-regr/report0.txt"
+  },
 #ifdef WITH_BTF
   {
     "data/test-abidiff-exit/btf/test0-v0.o",
