@@ -81,6 +81,7 @@ struct options
   bool				read_tu;
   bool				diff;
   bool				noout;
+  bool				annotate;
 #ifdef WITH_CTF
   bool				use_ctf;
 #endif
@@ -97,7 +98,8 @@ struct options
       read_from_stdin(false),
       read_tu(false),
       diff(false),
-      noout(false)
+      noout(false),
+      annotate(false)
 #ifdef WITH_CTF
     ,
       use_ctf(false)
@@ -483,24 +485,25 @@ display_usage(const string& prog_name, ostream& out)
   emit_prefix(prog_name, out)
     << "usage: " << prog_name << " [options] [<abi-file1>]\n"
     << " where options can be:\n"
-    << "  --help  display this message\n"
-    << "  --version|-v  display program version information and exit\n"
-    << "  --debug-info-dir <path> the path under which to look for "
-    "debug info for the elf <abi-file>\n"
-    << "  --headers-dir|--hd <path> the path to headers of the elf file\n"
-    << "  --header-file|--hf <path> the path to one header of the elf file\n"
-    << "  --suppressions|--suppr <path> specify a suppression file\n"
-    << "  --diff  for xml inputs, perform a text diff between "
-    "the input and the memory model saved back to disk\n"
-    << "  --noout  do not display anything on stdout\n"
-    << "  --stdin  read abi-file content from stdin\n"
-    << "  --tu  expect a single translation unit file\n"
+    << "  --annotate  annotate the ABI artifacts emitted in the output\n"
 #ifdef WITH_CTF
     << "  --ctf use CTF instead of DWARF in ELF files\n"
 #endif
+    << "  --debug-info-dir <path> the path under which to look for "
+    "debug info for the elf <abi-file>\n"
+    << "  --diff  for xml inputs, perform a text diff between "
+    "the input and the memory model saved back to disk\n"
+    << "  --header-file|--hf <path> the path to one header of the elf file\n"
+    << "  --headers-dir|--hd <path> the path to headers of the elf file\n"
+    << "  --help  display this message\n"
+    << "  --noout  do not display anything on stdout\n"
 #ifdef WITH_SHOW_TYPE_USE_IN_ABILINT
     << "  --show-type-use <type-id>  show how a type is used from the abixml file\n"
 #endif
+    << "  --stdin  read abi-file content from stdin\n"
+    << "  --suppressions|--suppr <path> specify a suppression file\n"
+    << "  --tu  expect a single translation unit file\n"
+    << "  --version|-v  display program version information and exit\n"
     ;
 }
 
@@ -583,6 +586,8 @@ parse_command_line(int argc, char* argv[], options& opts)
 	  opts.diff = true;
 	else if (!strcmp(argv[i], "--noout"))
 	  opts.noout = true;
+	else if (!strcmp(argv[i], "--annotate"))
+	  opts.annotate = true;
 #ifdef WITH_SHOW_TYPE_USE_IN_ABILINT
       else if (!strcmp(argv[i], "--show-type-use"))
 	{
@@ -723,6 +728,7 @@ main(int argc, char* argv[])
 	    {
 	      const write_context_sptr& ctxt
 		  = create_write_context(env, cout);
+	      set_annotate(*ctxt, opts.annotate);
 	      write_translation_unit(*ctxt, *tu, 0);
 	    }
 	  return 0;
@@ -739,6 +745,7 @@ main(int argc, char* argv[])
 	    {
 	      const write_context_sptr& ctxt
 		  = create_write_context(env, cout);
+	      set_annotate(*ctxt, opts.annotate);
 	      write_corpus(*ctxt, corp, /*indent=*/0);
 	    }
 	  return 0;
@@ -879,6 +886,7 @@ main(int argc, char* argv[])
 	    {
 	      if (!opts.noout)
 		{
+		  set_annotate(*ctxt, opts.annotate);
 		  if (corp)
 		    is_ok = write_corpus(*ctxt, corp, 0);
 		  else if (group)
